@@ -7,7 +7,7 @@
  *  within the code that implements the API.
  */
 
-// Copyright (C) 2014-2024 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2025 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef DDCUTIL_TYPES_H_
@@ -151,6 +151,7 @@ typedef enum {
    DDCA_STATS_ERRORS   = 0x02,    ///< error statistics
    DDCA_STATS_CALLS    = 0x04,    ///< system calls
    DDCA_STATS_ELAPSED  = 0x08,    ///< total elapsed time
+   DDCA_STATS_API      = 0x10,    ///< API specific stats
    DDCA_STATS_ALL      = 0xFF     ///< indicates all statistics types
 } DDCA_Stats_Type;
 
@@ -307,7 +308,7 @@ typedef struct {
    DDCA_IO_Mode io_mode;        ///< physical access mode
    union {
       int        i2c_busno;     ///< I2C bus number
-      int        hiddev_devno;  ///* USB hiddev device  number
+      int        hiddev_devno;  ///< USB hiddev device number
    } path;
 } DDCA_IO_Path;
 
@@ -339,6 +340,43 @@ typedef struct {
 } DDCA_Display_Info;
 
 
+#define DDCA_DRM_CONNECTOR_FIELD_SIZE 32
+
+typedef enum {
+   DDCA_DRM_CONNECTOR_NOT_FOUND,
+   DDCA_DRM_CONNECTOR_FOUND_BY_BUSNO,
+   DDCA_DRM_CONNECTOR_FOUND_BY_EDID
+} DDCA_Drm_Connector_Found_By;
+
+
+/** Describes one monitor detected by ddcutil.
+ *
+ *  This is an extended version of DDCA_Display_Info with additional information
+ *  and space allocated to allow for future extensions without breaking
+ *  the ABI.
+ *
+ *  This struct is copied to the caller and can simply be freed.
+ */
+typedef struct {
+   char                   marker[4];        ///< always "DDIN"
+   int                    dispno;           ///< ddcutil assigned display number
+   DDCA_IO_Path           path;             ///< physical access path to display
+   int                    usb_bus;          ///< USB bus number, if USB connection
+   int                    usb_device;       ///< USB device number, if USB connection
+   char                   mfg_id[    DDCA_EDID_MFG_ID_FIELD_SIZE    ]; ///< 3 character mfg id from EDID
+   char                   model_name[DDCA_EDID_MODEL_NAME_FIELD_SIZE]; ///< model name from EDID, 13 char max
+   char                   sn[        DDCA_EDID_SN_ASCII_FIELD_SIZE  ]; ///< "serial number" from EDID, 13 char max
+   uint16_t               product_code;     ///< model product number
+   uint8_t                edid_bytes[128];  ///< first 128 bytes of EDID
+   DDCA_MCCS_Version_Spec vcp_version;      ///< VCP version as pair of numbers
+   DDCA_Display_Ref       dref;             ///< opaque display reference
+   char                   drm_card_connector[DDCA_DRM_CONNECTOR_FIELD_SIZE];
+   DDCA_Drm_Connector_Found_By drm_card_connector_found_by;
+   int16_t                drm_connector_id;   ///< -1 if undefined in sysfs
+   void *                 unused[8];
+} DDCA_Display_Info2;
+
+
 /** Collection of #DDCA_Display_Info */
 typedef struct {
    int                ct;       ///< number of records
@@ -377,6 +415,7 @@ typedef uint16_t DDCA_Version_Feature_Flags;
 #define DDCA_STD_CONT     0x0080       /**< Normal continuous feature */
 #define DDCA_COMPLEX_CONT 0x0040       /**< Continuous feature with special interpretation */
 #define DDCA_SIMPLE_NC    0x0020       /**< Non-continuous feature, having a defined list of values in byte SL */
+#define DDCA_EXTENDED_NC  0x8000       /**< Like DDCA_SIMPLE_NC, but also using byte SH */
 #define DDCA_COMPLEX_NC   0x0010       /**< Non-continuous feature, having a complex interpretation using one or more of SL, SH, ML, MH */
 #define DDCA_NC_CONT      0x0800       /**< NC feature combining reserved values with continuous range */
 // For WO NC features.  There's no interpretation function or lookup table
@@ -386,7 +425,7 @@ typedef uint16_t DDCA_Version_Feature_Flags;
 #define DDCA_WO_TABLE     0x0002       /**< Write only table feature */
 
 #define DDCA_CONT         (DDCA_STD_CONT|DDCA_COMPLEX_CONT)            /**< Continuous feature, of any subtype */
-#define DDCA_NC           (DDCA_SIMPLE_NC|DDCA_COMPLEX_NC|DDCA_WO_NC|DDCA_NC_CONT)  /**< Non-continuous feature of any subtype */
+#define DDCA_NC           (DDCA_EXTENDED_NC |DDCA_SIMPLE_NC|DDCA_COMPLEX_NC|DDCA_WO_NC|DDCA_NC_CONT)  /**< Non-continuous feature of any subtype */
 #define DDCA_NON_TABLE    (DDCA_CONT | DDCA_NC)                        /**< Non-table feature of any type */
 
 #define DDCA_TABLE        (DDCA_NORMAL_TABLE | DDCA_WO_TABLE)        /**< Table type feature, of any subtype */
@@ -394,6 +433,8 @@ typedef uint16_t DDCA_Version_Feature_Flags;
 
 // Additional bits:
 #define DDCA_DEPRECATED   0x0001     /**< Feature is deprecated in the specified VCP version */
+// not really version specific:
+#define DDCA_NOVERIFY     0x8000     /**< Do not attempt to verify, verification known to fail */
 
 ///@}
 
@@ -531,12 +572,15 @@ typedef struct {
 //! Type of event being reported
 //!
 //!  @since 2.1.0
+//!
+//! As of ddcutil 2.2.0, events of type DDCA_EVENT_DPMS_AWAKE and
+//! DDCA_EVENT_DPMS_ASLEEP are no longer issued.
 typedef enum {
    DDCA_EVENT_DPMS_AWAKE,
    DDCA_EVENT_DPMS_ASLEEP,
    DDCA_EVENT_DISPLAY_CONNECTED,
    DDCA_EVENT_DISPLAY_DISCONNECTED,
-   DDCA_EVENT_UNUSED1,
+   DDCA_EVENT_DDC_ENABLED,
    DDCA_EVENT_UNUSED2,
 } DDCA_Display_Event_Type;
 
@@ -548,12 +592,12 @@ typedef enum {
 //!  @since 2.1.0
 typedef enum {
    DDCA_EVENT_CLASS_NONE               = 0,
-   DDCA_EVENT_CLASS_DPMS               = 1,
+   DDCA_EVENT_CLASS_DPMS               = 1,   // unimplemented as of 2.2.0
    DDCA_EVENT_CLASS_DISPLAY_CONNECTION = 2,
    DDCA_EVENT_CLASS_UNUSED1            = 4,
 } DDCA_Display_Event_Class;
 
-#define DDCA_EVENT_CLASS_ALL (DDCA_EVENT_CLASS_DPMS | DDCA_EVENT_CLASS_DISPLAY_CONNECTION)
+#define DDCA_EVENT_CLASS_ALL ((DDCA_Display_Event_Class)(DDCA_EVENT_CLASS_DPMS | DDCA_EVENT_CLASS_DISPLAY_CONNECTION))
 
 
 /** Event record passed by a display status callback function.
@@ -565,18 +609,25 @@ typedef enum {
  *  maximum of 31 printable characters and a termination byte.
  *
  *  @remark
- *  The DDCA_Display_Status_Event is defined with two unused fields to allow
+ *  The DDCA_Display_Status_Event is defined with unused fields at the end to allow
  *  for future extension without breaking the ABI.
  *
  *  @since 2.1.0
+ *
+ *  @remark
+ *  Field flags with bit DDCA_DISPLAY_EVENT_DDC_WORKING added in 2.2.0
  */
+
+#define DDCA_DISPLAY_EVENT_DDC_WORKING 0x08
+
 typedef struct {
    uint64_t                timestamp_nanos;
    DDCA_Display_Event_Type event_type;
    DDCA_IO_Path            io_path;
    char                    connector_name[32];
    DDCA_Display_Ref        dref;
-   void *                  unused[2];
+   uint8_t                 flags;
+   void *                  unused[1];
 } DDCA_Display_Status_Event;
 
 
@@ -599,6 +650,32 @@ void (*DDCA_Display_Status_Callback_Func)(DDCA_Display_Status_Event event);
 
 
 
+/** Parameter record for tuning watching display connection changes.
+ *
+ *  @remark
+ *  This struct is defined with unused fields at the end to allow for future
+ *  extension without breaking the ABI.
+ *
+ *  @since 2.2.0
+ */
+//! @remark see init_display_watch_options() for consideration of what to add
+typedef struct {
+   // Polling loop intervals for each of the watch modes
+   uint16_t   xevent_watch_interval_millisec;   /**< For watch mode XEVENT */
+   uint16_t   poll_watch_interval_millisec;     /**< For watch mode POLL */
+
+   // Once an event is received that possibly indicates a display change, libddcutil
+   // repeatedly checks /sys/class/drm until the reported displays stabilize
+   uint16_t  initial_stabilization_millisec; /**< Delay before first_initialization check */
+   uint16_t  stabilization_poll_millisec;    /**< Polling interval between stabilization checks */
+
+   // Interval at which to check that DDC has become enabled if it is not
+   // immediately enabled when the EDID is detected.
+   uint16_t   watch_retry_interval_millisec;
+
+   void *  unused[4];
+} DDCA_DW_Settings;
+
 #ifdef __cplusplus
 }
 #endif
